import 'package:flight_script/flight_script.dart';

/// Dart functions can also affect the behaviour of the interpretter
/// here we define a new comment syntax, by telling the compiler to ignore everything until
/// the matching identifier
///
/// We can also implement something that looks like heredocs in ruby
/// Do not abuse this power
const program = """
  <!-- this is a comment -->
  
  <<~
  This is a Heredoc style string. \n
  There is really no good reason to do this.
  >>
  
""";

void main() {
  final interpreter = Interpreter();

  interpreter.bind("<!--", (vm) {
    vm.deferUntil("-->", (_vm, _values) {});
  });

  interpreter.bind("<<~", (vm) {
    vm.deferUntil(
        ">>", (vm, values) => vm.push(values.map((e) => e.value).join(" ")));
  });

  final result = interpreter.eval(program);
  print(result);
}
