import 'virtual_machine.dart';

/// Command to pop a value off the stack and return it
class Pop {
  const Pop();

  @override
  String toString() => "pop";
}

/// Duplicate the top value on the stack and add it
class Duplicate {
  const Duplicate();

  @override
  String toString() => "dup";
}

/// Rotate the stack at a given depth
class Rotate {
  final int depth;
  final int steps;

  const Rotate(this.depth, this.steps)
      : assert(depth > 1),
        assert(steps > 0);

  @override
  String toString() => "rot($depth, $steps)";
}

/// Push a value onto the stack
/// If a value is not recognised as a command, it is pushed, but we need this
/// wrapper to explicitly push a value
class Push {
  final dynamic value;

  Push(this.value) : assert(value != null);

  @override
  String toString() => "push($value)";
}

/// just fetch the whole stack - mostly a debug method
class Stack {
  const Stack();
  @override
  String toString() => "stack";
}

/// the stack is maintained at the lowest level of FlightVM, with
/// all values pushed, except specific commands which pop
class StackListener extends ValueListener {
  final stack = [];

  @override
  invoke(VirtualMachine vm, dynamic value, ValueCallback next) {
    if (value is Pop) {
      return stack.removeLast();
    }

    if (value is Duplicate) {
      stack.add(stack.last);
      return;
    }

    if (value is Rotate) {
      for (int i = 0; i < value.steps; i++) {
        stack.add(stack.removeAt(stack.length - value.depth));
      }
      return;
    }

    if (value is Stack) return stack;

    if (value is Push) {
      stack.add(value.value);
      return;
    }

    // if all else fails, push the value anyway
    stack.add(value);
    return;
  }
}

extension StackApi on VirtualMachine {
  void push(dynamic value) => eval(Push(value));

  dynamic pop() => eval(const Pop());

  dynamic duplicate() => eval(const Duplicate());

  dynamic rotate(int depth, int steps) => eval(Rotate(depth, steps));

  dynamic swap() => eval(const Rotate(2, 1));
}
