import 'dart:collection';

/// callback to execute with a value
typedef ValueCallback = dynamic Function(dynamic);

/// Specifies what to do when a value is received
/// The next ValueListener in the listener stack is passed, so that the listeners may be chained
/// if desired
abstract class ValueListener {
  dynamic invoke(VirtualMachine vm, dynamic value, ValueCallback next);
}

/// The virtual machine is actually just a set of listeners that are combined and called
/// when a command is received
/// Because I'm a terrible programmer we have a cyclc dependency here - could be broken
/// with a VM interface but I'm lazy today
class VirtualMachine {
  final _listeners = Queue<ValueListener>();

  void pushListener(ValueListener listener) => _listeners.addLast(listener);

  bool removeListener(ValueListener listener) => _listeners.remove(listener);

  ValueListener popListener() => _listeners.removeLast();

  bool hasListener(Type listener) =>
      _listeners.any((element) => element.runtimeType == listener);

  dynamic eval(dynamic value) => _listeners.fold<ValueCallback>(
      (command) => command,
      (previous, listener) => (c) => listener.invoke(this, c, previous))(value);
}
