import 'package:flight_script/src/dictionary.dart';
import 'package:flight_script/src/stack.dart';
import 'package:flight_script/src/virtual_machine.dart';
import 'package:test/test.dart';

void main() {
  test('The empty vm returns passed values when called', () {
    final vm = VirtualMachine();
    final result = vm.eval(7.0);
    expect(result, equals(7.0));
  });

  group('stack', () {
    late VirtualMachine vm;

    setUp(() {
      vm = VirtualMachine();
      vm.pushListener(StackListener());
    });

    test('can push and pop', () {
      vm.push(7.0);
      vm.push("Hello World");

      expect(vm.pop(), equals("Hello World"));
      expect(vm.pop(), equals(7.0));
    });

    test('can duplicate', () {
      vm.push("Hello");
      vm.duplicate();

      expect(vm.pop(), equals("Hello"));
      expect(vm.pop(), equals("Hello"));
    });

    test('can rotate', () {
      vm.push("Hello");
      vm.push("!");
      vm.push("World");
      vm.push(",");
      vm.rotate(4, 1);

      expect(vm.pop(), equals("Hello"));
      expect(vm.pop(), equals(","));
      expect(vm.pop(), equals("World"));
      expect(vm.pop(), equals("!"));
    });
  });

  test('Identifier lookup works and the identifier pushes the value', () {
    final vm = VirtualMachine();
    vm.pushListener(StackListener());
    vm.pushListener(DictionaryListener());

    vm["foo"] = "bar";
    vm["foo"];

    final result = vm.pop();

    expect(result, equals("bar"));
  });

  test('Identifier lookup failure gives an error', () {
    final vm = VirtualMachine();
    vm.pushListener(StackListener());
    vm.pushListener(DictionaryListener());

    expect(() => vm["foo"], throwsStateError);
  });

  group('macro', () {
    test('building a macro and calling it works', () {});
  });
}
